@@ -6,23 +6,24 @@
 #   copyright and license terms.
 #
 ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
-''' Read / write access to CIfTI2 image format
+''' Read / write access to CIFTI-2 image format
 
 Format of the NIFTI2 container format described here:
 
     http://www.nitrc.org/forum/message.php?msg_id=3738
 
-Definition of the CIFTI2 header format and file extensions attached to this
-email:
+Definition of the CIFTI-2 header format and file extensions can be found at:
 
-    http://www.nitrc.org/forum/forum.php?thread_id=4380&forum_id=1955
-
-Filename is ``CIFTI-2_Main_FINAL_1March2014.pdf``.
+    http://www.nitrc.org/projects/cifti
 '''
 from __future__ import division, print_function, absolute_import
 import re
-import collections
-
+try:
+    from collections.abc import MutableSequence, MutableMapping, Iterable
+except ImportError:
+    # PY2 compatibility
+    from collections import MutableSequence, MutableMapping, Iterable
+from collections import OrderedDict
 from .. import xmlutils as xml
 from ..filebasedimages import FileBasedHeader
 from ..dataobj_images import DataobjImage
@@ -38,7 +39,7 @@ def _float_01(val):
 
 
 class Cifti2HeaderError(Exception):
-    """ Error in CIFTI2 header
+    """ Error in CIFTI-2 header
     """
 
 
@@ -104,7 +105,7 @@ def _underscore(string):
     return re.sub(r'([a-z0-9])([A-Z])', r'\1_\2', string).lower()
 
 
-class Cifti2MetaData(xml.XmlSerializable, collections.MutableMapping):
+class Cifti2MetaData(xml.XmlSerializable, MutableMapping):
     """ A list of name-value pairs
 
     * Description - Provides a simple method for user-supplied metadata that
@@ -124,7 +125,7 @@ class Cifti2MetaData(xml.XmlSerializable, collections.MutableMapping):
     data : list of (name, value) tuples
     """
     def __init__(self, metadata=None):
-        self.data = collections.OrderedDict()
+        self.data = OrderedDict()
         if metadata is not None:
             self.update(metadata)
 
@@ -173,8 +174,8 @@ class Cifti2MetaData(xml.XmlSerializable, collections.MutableMapping):
         return metadata
 
 
-class Cifti2LabelTable(xml.XmlSerializable, collections.MutableMapping):
-    """ CIFTI2 label table: a sequence of ``Cifti2Label``s
+class Cifti2LabelTable(xml.XmlSerializable, MutableMapping):
+    """ CIFTI-2 label table: a sequence of ``Cifti2Label``s
 
     * Description - Used by NamedMap when IndicesMapToDataType is
       "CIFTI_INDEX_TYPE_LABELS" in order to associate names and display colors
@@ -191,7 +192,7 @@ class Cifti2LabelTable(xml.XmlSerializable, collections.MutableMapping):
     """
 
     def __init__(self):
-        self._labels = collections.OrderedDict()
+        self._labels = OrderedDict()
 
     def __len__(self):
         return len(self._labels)
@@ -232,7 +233,7 @@ class Cifti2LabelTable(xml.XmlSerializable, collections.MutableMapping):
 
 
 class Cifti2Label(xml.XmlSerializable):
-    """ CIFTI2 label: association of integer key with a name and RGBA values
+    """ CIFTI-2 label: association of integer key with a name and RGBA values
 
     For all color components, value is floating point with range 0.0 to 1.0.
 
@@ -283,7 +284,7 @@ class Cifti2Label(xml.XmlSerializable):
         return (self.red, self.green, self.blue, self.alpha)
 
     def _to_xml_element(self):
-        if self.label is '':
+        if self.label == '':
             raise Cifti2HeaderError('Label needs a name')
         try:
             v = int(self.key)
@@ -310,7 +311,7 @@ class Cifti2Label(xml.XmlSerializable):
 
 
 class Cifti2NamedMap(xml.XmlSerializable):
-    """CIFTI2 named map: association of name and optional data with a map index
+    """CIFTI-2 named map: association of name and optional data with a map index
 
     Associates a name, optional metadata, and possibly a LabelTable with an
     index in a map.
@@ -427,8 +428,8 @@ class Cifti2Surface(xml.XmlSerializable):
         return surf
 
 
-class Cifti2VoxelIndicesIJK(xml.XmlSerializable, collections.MutableSequence):
-    """CIFTI2 VoxelIndicesIJK: Set of voxel indices contained in a structure
+class Cifti2VoxelIndicesIJK(xml.XmlSerializable, MutableSequence):
+    """CIFTI-2 VoxelIndicesIJK: Set of voxel indices contained in a structure
 
     * Description - Identifies the voxels that model a brain structure, or
       participate in a parcel. Note that when this is a child of BrainModel,
@@ -509,8 +510,8 @@ class Cifti2VoxelIndicesIJK(xml.XmlSerializable, collections.MutableSequence):
         return vox_ind
 
 
-class Cifti2Vertices(xml.XmlSerializable, collections.MutableSequence):
-    """CIFTI2 vertices - association of brain structure and a list of vertices
+class Cifti2Vertices(xml.XmlSerializable, MutableSequence):
+    """CIFTI-2 vertices - association of brain structure and a list of vertices
 
     * Description - Contains a BrainStructure type and a list of vertex indices
       within a Parcel.
@@ -576,7 +577,7 @@ class Cifti2Vertices(xml.XmlSerializable, collections.MutableSequence):
 
 
 class Cifti2Parcel(xml.XmlSerializable):
-    """CIFTI2 parcel: association of a name with vertices and/or voxels
+    """CIFTI-2 parcel: association of a name with vertices and/or voxels
 
     * Description - Associates a name, plus vertices and/or voxels, with an
       index.
@@ -691,7 +692,7 @@ class Cifti2TransformationMatrixVoxelIndicesIJKtoXYZ(xml.XmlSerializable):
 
 
 class Cifti2Volume(xml.XmlSerializable):
-    """CIFTI2 volume: information about a volume for mappings that use voxels
+    """CIFTI-2 volume: information about a volume for mappings that use voxels
 
     * Description - Provides information about the volume for any mappings that
       use voxels.
@@ -733,8 +734,8 @@ class Cifti2Volume(xml.XmlSerializable):
         return volume
 
 
-class Cifti2VertexIndices(xml.XmlSerializable, collections.MutableSequence):
-    """CIFTI2 vertex indices: vertex indices for an associated brain model
+class Cifti2VertexIndices(xml.XmlSerializable, MutableSequence):
+    """CIFTI-2 vertex indices: vertex indices for an associated brain model
 
     The vertex indices (which are independent for each surface, and
     zero-based) that are used in this brain model[.] The parent
@@ -889,7 +890,7 @@ class Cifti2BrainModel(xml.XmlSerializable):
         return brain_model
 
 
-class Cifti2MatrixIndicesMap(xml.XmlSerializable, collections.MutableSequence):
+class Cifti2MatrixIndicesMap(xml.XmlSerializable, MutableSequence):
     """Class for Matrix Indices Map
 
     * Description - Provides a mapping between matrix indices and their
@@ -1076,8 +1077,8 @@ class Cifti2MatrixIndicesMap(xml.XmlSerializable, collections.MutableSequence):
         return mat_ind_map
 
 
-class Cifti2Matrix(xml.XmlSerializable, collections.MutableSequence):
-    """ CIFTI2 Matrix object
+class Cifti2Matrix(xml.XmlSerializable, MutableSequence):
+    """ CIFTI-2 Matrix object
 
     This is a list-like container where the elements are instances of
     :class:`Cifti2MatrixIndicesMap`.
@@ -1122,7 +1123,7 @@ class Cifti2Matrix(xml.XmlSerializable, collections.MutableSequence):
         applies_to_matrix_dimension = mim.applies_to_matrix_dimension
         if not isinstance(
             applies_to_matrix_dimension,
-            collections.Iterable
+            Iterable
         ):
             applies_to_matrix_dimension = (int(applies_to_matrix_dimension),)
         return applies_to_matrix_dimension
@@ -1209,7 +1210,7 @@ class Cifti2Matrix(xml.XmlSerializable, collections.MutableSequence):
 
 
 class Cifti2Header(FileBasedHeader, xml.XmlSerializable):
-    ''' Class for CIFTI2 header extension '''
+    ''' Class for CIFTI-2 header extension '''
 
     def __init__(self, matrix=None, version="2.0"):
         FileBasedHeader.__init__(self)
@@ -1264,9 +1265,43 @@ class Cifti2Header(FileBasedHeader, xml.XmlSerializable):
         '''
         return self.matrix.get_index_map(index)
 
+    def get_axis(self, index):
+        '''
+        Generates the Cifti2 axis for a given dimension
+
+        Parameters
+        ----------
+        index : int
+            Dimension for which we want to obtain the mapping.
+
+        Returns
+        -------
+        axis : :class:`.cifti2_axes.Axis`
+        '''
+        from . import cifti2_axes
+        return cifti2_axes.from_index_mapping(self.matrix.get_index_map(index))
+
+    @classmethod
+    def from_axes(cls, axes):
+        '''
+        Creates a new Cifti2 header based on the Cifti2 axes
+
+        Parameters
+        ----------
+        axes : tuple of :class`.cifti2_axes.Axis`
+            sequence of Cifti2 axes describing each row/column of the matrix to be stored
+
+        Returns
+        -------
+        header : Cifti2Header
+            new header describing the rows/columns in a format consistent with Cifti2
+        '''
+        from . import cifti2_axes
+        return cifti2_axes.to_header(axes)
+
 
 class Cifti2Image(DataobjImage):
-    """ Class for single file CIFTI2 format image
+    """ Class for single file CIFTI-2 format image
     """
     header_class = Cifti2Header
     valid_exts = Nifti2Image.valid_exts
@@ -1293,8 +1328,10 @@ class Cifti2Image(DataobjImage):
             Object containing image data.  It should be some object that
             returns an array from ``np.asanyarray``.  It should have a
             ``shape`` attribute or property.
-        header : Cifti2Header instance
-            Header with data for / from XML part of CIFTI2 format.
+        header : Cifti2Header instance or sequence of :class:`cifti2_axes.Axis`
+            Header with data for / from XML part of CIFTI-2 format.
+            Alternatively a sequence of cifti2_axes.Axis objects can be provided
+            describing each dimension of the array.
         nifti_header : None or mapping or NIfTI2 header instance, optional
             Metadata for NIfTI2 component of this format.
         extra : None or mapping
@@ -1302,6 +1339,8 @@ class Cifti2Image(DataobjImage):
         file_map : mapping, optional
             Mapping giving file information for this image format.
         '''
+        if not isinstance(header, Cifti2Header) and header:
+            header = Cifti2Header.from_axes(header)
         super(Cifti2Image, self).__init__(dataobj, header=header,
                                           extra=extra, file_map=file_map)
         self._nifti_header = Nifti2Header.from_header(nifti_header)
@@ -1317,7 +1356,7 @@ class Cifti2Image(DataobjImage):
 
     @classmethod
     def from_file_map(klass, file_map):
-        """ Load a CIFTI2 image from a file_map
+        """ Load a CIFTI-2 image from a file_map
 
         Parameters
         ----------
@@ -1337,7 +1376,7 @@ class Cifti2Image(DataobjImage):
                 cifti_header = item.get_content()
                 break
         else:
-            raise ValueError('NIfTI2 header does not contain a CIFTI2 '
+            raise ValueError('NIfTI2 header does not contain a CIFTI-2 '
                              'extension')
 
         # Construct cifti image.
@@ -1396,7 +1435,7 @@ class Cifti2Image(DataobjImage):
         img.to_file_map(file_map or self.file_map)
 
     def update_headers(self):
-        ''' Harmonize CIFTI2 and NIfTI headers with image data
+        ''' Harmonize CIFTI-2 and NIfTI headers with image data
 
         >>> import numpy as np
         >>> data = np.zeros((2,3,4))
